local debug = true

local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
	insets = {left = 4, right = 4, top = 4, bottom = 4},
}

local function TakeAction(frame)
  if "auction" == frame.tiptext then
    HazLoot:OpenNext()
  elseif "offspec" == frame.tiptext then
    HazLoot:OpenToRoll()
  elseif "last call" == frame.tiptext then
    HazLoot:LastCall()
  elseif "bank" == frame.tiptext then
    HazLoot:BankCurrentItem()
  elseif "dust" == frame.tiptext then
    HazLoot:DustCurrentItem()
  elseif "confirm" == frame.tiptext then
    HazLoot:AwardCurrentItem()
  else
    print("HLUI: unknown button: "..(frame.tiptext or "nil tiptext"))
  end
end

local function HideTip() GameTooltip:Hide() end
local function HideTip2() GameTooltip:Hide(); ResetCursor() end

local function SetTip(frame)
	GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
	GameTooltip:SetText(frame.tiptext) -- TODO: Make these tooltips more useful
	GameTooltip:Show()
end

local function SetItemTip(frame)
	if not frame.link then return end
	GameTooltip:SetOwner(frame, "ANCHOR_LEFT")
	GameTooltip:SetHyperlink(frame.link)
	if IsShiftKeyDown() then GameTooltip_ShowCompareItem() end
	CursorUpdate()
end

local function ItemOnUpdate(self)
	if IsShiftKeyDown() then GameTooltip_ShowCompareItem() end
	CursorOnUpdate(self)
end

local function LootClick(frame)
  print("clicked: "..(frame:GetName() or frame:GetParent():GetName() or "anonymous frame"))
	if IsControlKeyDown() then DressUpItemLink(frame.link)
	elseif IsShiftKeyDown() then ChatEdit_InsertLink(frame.link)
  else
    HazLoot:AnnouncePhatLewt()
  end
end

local function something(frame)
end

local function CreateActionButton(parent, ntex, ptex, htex, tiptext, ...)
	local f = CreateFrame("Button", nil, parent)
	f:SetPoint(...)
	f:SetWidth(28)
	f:SetHeight(28)
	f:SetNormalTexture(ntex)
	if ptex then f:SetPushedTexture(ptex) end
	f:SetHighlightTexture(htex)
	f.parent = parent
	f.tiptext = tiptext
	f:SetScript("OnEnter", SetTip)
	f:SetScript("OnLeave", HideTip)
	f:SetScript("OnClick", TakeAction)
	local txt = f:CreateFontString(nil, nil, "GameFontHighlightSmallOutline")
	txt:SetPoint("CENTER", 0, 0)
	return f, txt
end

local function CreateCandidateFrame()
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetWidth(200)
	frame:SetHeight(40)
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(0, 0, 0, .7)
	frame:SetBackdropBorderColor(0.7, 0.7, 0, 1)

	local confirm, confirmtext = CreateActionButton(frame, "Interface\\Buttons\\UI-CheckBox-Check", "Interface\\Buttons\\UI-CheckBox-Down", "Interface\\Buttons\\UI-CheckBox-Check", "confirm", "TOPLEFT", frame, "TOPLEFT", 12, -5)
  frame.confirm = confirmtext
	frame.confirm:SetText("confirm")

	local winner = frame:CreateFontString()
	winner:SetPoint("TOPLEFT", confirm, "TOPRIGHT", 5, -13)
	winner:SetFont("Fonts\\FRIZQT__.TTF", 13, "THICKOUTLINE")
	winner:SetJustifyH("LEFT")
	frame.winner = winner

  local losers = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
  losers:SetPoint("TOPLEFT", winner, "BOTTOMLEFT", 2, 5)
	losers:SetFont("Fonts\\FRIZQT__.TTF", 10, "THINOUTLINE")
	losers:SetJustifyH("LEFT")
	frame.losers = losers

  return frame
end

local function CreatePlayerDisplayFrame()
	local frame = CreateFrame("Frame", "HLUI_PlayerDisplay", UIParent)
	frame:SetWidth(200)
	frame:SetHeight(110)
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(0, 0, 0, .7)
	frame:SetBackdropBorderColor(.7, 0, 0, 1)

	local title = frame:CreateFontString()
	title:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5)
	title:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -5, -5)
	title:SetFont("Fonts\\FRIZQT__.TTF", 13, "THICKOUTLINE")
  title:SetJustifyH("CENTER")
	frame.title = title

  return frame
end

local function CreateControlFrame()
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetWidth(300)
	frame:SetHeight(110)
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(0, 0, 0, .7)

	local button = CreateFrame("Button", nil, frame)
	button:SetPoint("TOPLEFT", -18, 18)
	button:SetWidth(36)
	button:SetHeight(36)
	button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
	button:GetHighlightTexture():SetBlendMode("ADD")
	button:SetScript("OnEnter", SetItemTip)
	button:SetScript("OnLeave", HideTip2)
	button:SetScript("OnUpdate", ItemOnUpdate)
	-- button:SetScript("OnClick", LootClick)
	frame.button = button

	local buttonborder = CreateFrame("Frame", nil, button)
	buttonborder:SetWidth(44)
	buttonborder:SetHeight(44)
	buttonborder:SetPoint("CENTER", button, "CENTER")
	buttonborder:SetBackdrop(backdrop)
	buttonborder:SetBackdropColor(1, 1, 1, 0)
	frame.buttonborder = buttonborder

	local auction, auctiontext = CreateActionButton(frame, "Interface\\TargetingFrame\\UI-TargetingFrame-Skull", "Interface\\Buttons\\UI-GroupLoot-Coin-Highlight", "Interface\\Buttons\\UI-GroupLoot-Coin-Down", "auction", "TOPLEFT", frame.button, "TOPRIGHT", 135, -59)
	local auc_lc, auc_lctext = CreateActionButton(frame, "Interface\\TargetingFrame\\UI-TargetingFrame-Skull", "Interface\\Buttons\\UI-GroupLoot-Coin-Highlight", "Interface\\Buttons\\UI-GroupLoot-Coin-Down", "last call", "TOPLEFT", auction, "BOTTOMLEFT", 0, -5)
	local offspec, offspectext = CreateActionButton(frame, "Interface\\Buttons\\UI-GroupLoot-Dice-Up", "Interface\\Buttons\\UI-GroupLoot-Dice-Highlight", "Interface\\Buttons\\UI-GroupLoot-Dice-Down", "offspec", "TOPLEFT", auction, "TOPRIGHT", 25, 0)
	local bank, banktext = CreateActionButton(frame, "Interface\\Buttons\\UI-GroupLoot-Coin-Up", nil, "Interface\\Buttons\\UI-GroupLoot-Coin-Down", "bank", "LEFT", offspec, "RIGHT", 23, -2)
	local dust, dusttext = CreateActionButton(frame, "Interface\\Buttons\\UI-GroupLoot-Pass-Up", nil, "Interface\\Buttons\\UI-GroupLoot-Pass-Down", "dust", "TOP", bank, "BOTTOM", 0, 0)
	frame.auction, frame.auc_lc, frame.offspec, frame.bank, frame.dust = auctiontext, auc_lctext, offspectext, banktext, dusttext

	local bind = frame:CreateFontString()
	bind:SetPoint("TOPLEFT", button, "BOTTOMRIGHT", 4, 7)
	bind:SetFont("Fonts\\FRIZQT__.TTF", 13, "THICKOUTLINE")
	frame.fsbind = bind

  local roles = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
  roles:SetPoint("TOPLEFT", bind, "BOTTOMLEFT", -15, -5)
	roles:SetPoint("BOTTOMRIGHT", auc_lc, "BOTTOMLEFT", -5, 0)
	frame.fsroles = roles

	local loot = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	loot:SetPoint("LEFT", bind, "RIGHT", 0, .12)
	loot:SetPoint("RIGHT", frame, "RIGHT", -5, 0)
	loot:SetHeight(16)
	loot:SetJustifyH("LEFT")
	frame.fsloot = loot

	return frame
end

local function CreateLootFrame()
	local frame = CreateFrame("Frame", nil, UIParent)
	frame:SetWidth(320)
	frame:SetHeight(26)
	frame:SetBackdrop(backdrop)
	frame:SetBackdropColor(0, 0, 0, .9)

	local button = CreateFrame("Button", nil, frame)
	button:SetPoint("LEFT", 5, 0)
	button:SetWidth(24)
	button:SetHeight(24)
	button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
	button:GetHighlightTexture():SetBlendMode("ADD")
	button:SetScript("OnEnter", SetItemTip)
	button:SetScript("OnLeave", HideTip2)
	button:SetScript("OnUpdate", ItemOnUpdate)
	-- button:SetScript("OnClick", LootClick)
	frame.button = button

	local buttonborder = CreateFrame("Frame", nil, button)
	buttonborder:SetWidth(32)
	buttonborder:SetHeight(32)
	buttonborder:SetPoint("CENTER", button, "CENTER")
	buttonborder:SetBackdrop(backdrop)
	buttonborder:SetBackdropColor(1, 1, 1, 0)
	frame.buttonborder = buttonborder

	local bind = frame:CreateFontString()
	bind:SetPoint("TOPLEFT", button, "TOPRIGHT", 4, -6)
	bind:SetFont("Fonts\\FRIZQT__.TTF", 13, "THICKOUTLINE")
	frame.fsbind = bind

	local loot = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	loot:SetPoint("LEFT", bind, "RIGHT", 0, .12)
	-- loot:SetHeight(16)
	loot:SetJustifyH("LEFT")
	frame.fsloot = loot

  local roles = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	roles:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -6, 8)
  roles:SetText("Roles")
	-- loot:SetHeight(16)
	roles:SetJustifyH("RIGHT")
	frame.fsroles = roles

	return frame
end

local helpAnchor = CreateFrame("Button", "HLUI_HelpAnchor", UIParent)
helpAnchor:SetWidth(278) helpAnchor:SetHeight(20)
helpAnchor:SetBackdrop(backdrop)
helpAnchor:SetBackdropColor(0.25, 0.25, 0.25, .7)
local label = helpAnchor:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
label:SetAllPoints(helpAnchor)
label:SetText("HazLoot")
helpAnchor.label = label
helpAnchor:SetScript("OnClick", LootClick)
helpAnchor:SetScript("OnDragStart", helpAnchor.StartMoving)
helpAnchor:SetScript("OnDragStop", function(self)
	self:StopMovingOrSizing()
	hazLootDB.db.x, hazLootDB.db.y = self:GetCenter()
end)
helpAnchor:SetMovable(true)
helpAnchor:EnableMouse(true)
helpAnchor:RegisterForDrag("LeftButton")
helpAnchor:RegisterForClicks("LeftButtonUp")
helpAnchor:Hide()

HLUI = CreateFrame("Frame", "HLUI")

local lootFrames = {}
local wonLootFrames = {}
local controlFrame = CreateControlFrame()
local candidateFrame = CreateCandidateFrame()

local function GetFrame(loot)
  for i,f in ipairs(lootFrames) do
    if not loot.winner and not f.loot then return f end
  end
  for i,f in ipairs(wonLootFrames) do
    if loot.winner and not f.loot then return f end
  end

  local f = CreateLootFrame()
  if not loot.winner then
    f:SetPoint("BOTTOMLEFT", next(lootFrames) and lootFrames[#lootFrames] or helpAnchor, "TOPLEFT", next(lootFrames) and -12 or -60, 3)
    table.insert(lootFrames, f)
  else
    f:SetPoint("TOPLEFT", next(wonLootFrames) and wonLootFrames[#wonLootFrames] or controlFrame, "BOTTOMLEFT", next(wonLootFrames) and 0 or -3, -4)
    table.insert(wonLootFrames, f)
  end
	return f
end

local playerFrames = {}
local playerFrame = CreatePlayerDisplayFrame()
playerFrame:SetPoint("TOPRIGHT",helpAnchor,"TOPLEFT", -50, 0)
playerFrame:Hide()

local function CreatePlayerFrame()
  local frame = CreateFrame("Button", nil, UIParent) -- add activity later
  frame:SetWidth(200)
  frame:SetHeight(12)

	local name = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
	name:SetAllPoints(frame)
  name:SetJustifyH("LEFT")
	frame.name = name

  return frame
end

local function GetPlayerFrame(player)
  for i,f in ipairs(playerFrames) do
    if not f.player then return f end
  end

  local f = CreatePlayerFrame()
  if next(playerFrames) then
    f:SetPoint("TOPLEFT", playerFrames[#playerFrames], "BOTTOMLEFT", 0, -2)
  else
    f:SetPoint("TOPLEFT", playerFrame, "TOPLEFT", 10, -25)
  end
  table.insert(playerFrames, f)
  return f
end

local function CreatePlayerListEntry(i, player)
  f = GetPlayerFrame(player)    -- add to playerFrame: name, presentPos, masterPos, won
  f.player = player
  for mPos,name in pairs(player) do
    f.name:SetText(i..". "..(name or 'nil').." ["..mPos.."]")
  end
  f:Show()
end

local function PlayerDisplay(eligible)
  playerFrame.title:SetText("Present")
  playerFrame:SetPoint("TOPRIGHT",helpAnchor,"BOTTOMLEFT", -50, -2)
  local f
  for i,player in ipairs(eligible) do
    CreatePlayerListEntry(i, player)
  end
  playerFrame:SetHeight(30 + (14 * #eligible) +5)
  playerFrame:Show()
end

local function ControlHazLootItem(loot)
	local f = controlFrame
	f.loot = loot
	f.auction:SetText("suicide")
	f.auc_lc:SetText("last call")
	f.offspec:SetText("offspec")
	f.bank:SetText("bank")
	f.dust:SetText("dust")
	f.button:SetNormalTexture(loot.tex)
	f.button.link = loot.link

	f.fsbind:SetText(loot.bop and "BoP" or "BoE")
	f.fsbind:SetVertexColor(loot.bop and 1 or .3, loot.bop and .3 or 1, loot.bop and .1 or .3)

  f.fsroles:SetText(loot.roles)

	local color = ITEM_QUALITY_COLORS[loot.q]
	f.fsloot:SetVertexColor(color.r, color.g, color.b)
	f.fsloot:SetText(loot.name)

	f:SetBackdropBorderColor(color.r, color.g, color.b, 1)
	f.buttonborder:SetBackdropBorderColor(color.r, color.g, color.b, 1)
  f:SetPoint("TOPLEFT",helpAnchor,"BOTTOMLEFT", -22, -2)
	f:Show()
end

local function CreateHazLootItem(loot)
	local f = GetFrame(loot)
	f.loot = loot
	f.button:SetNormalTexture(loot.tex)
	f.button.link = loot.link
	f.fsbind:SetText(loot.bop and "BoP" or "BoE")
	f.fsbind:SetVertexColor(loot.bop and 1 or .3, loot.bop and .3 or 1, loot.bop and .1 or .3)
	local color = ITEM_QUALITY_COLORS[loot.q]
	f.fsloot:SetVertexColor(color.r, color.g, color.b)
	f.fsloot:SetText(loot.name)
  f.fsroles:SetText(loot.roles)
	f:SetBackdropBorderColor(color.r, color.g, color.b, 1)
	f.buttonborder:SetBackdropBorderColor(color.r, color.g, color.b, 1)
	f:Show()
end

local function CreateHazLootPassim(loot)
	local f = GetFrame(loot)
	f.loot = loot
	f.button:SetNormalTexture(loot.tex)
	f.button.link = loot.link
	f.fsbind:SetText(loot.bop and "BoP" or "BoE")
	f.fsbind:SetVertexColor(loot.bop and 1 or .3, loot.bop and .3 or 1, loot.bop and .1 or .3)
	local color = ITEM_QUALITY_COLORS[0]
	f.fsloot:SetVertexColor(color.r, color.g, color.b)
	f.fsloot:SetText(loot.name)
  f.fsroles:SetText(loot.winner)
	f:SetBackdropBorderColor(color.r, color.g, color.b, 1)
	f.buttonborder:SetBackdropBorderColor(color.r, color.g, color.b, 1)
	f:Show()
end

function HLUI:ShowLootList(lootTable, eligible)
  if lootTable then
    HLUI:CloseLoot()
    local shownControl = 0
    for i,item in ipairs(lootTable) do
      if not item.winner and shownControl == 0 then
        shownControl = i
        ControlHazLootItem(lootTable[i])
      elseif shownControl > 0 and i > shownControl then
        CreateHazLootItem(lootTable[i])
      else
        CreateHazLootPassim(lootTable[i])
      end
    end
    PlayerDisplay(eligible)
    helpAnchor:Show()

  end
end

function HLUI:CloseLoot()
  helpAnchor:Hide()
  controlFrame:Hide()
  candidateFrame:Hide()
  playerFrame:Hide()
  for i,frame in ipairs(lootFrames) do
    frame.loot = nil
    frame:Hide()
  end
  for i,frame in ipairs(wonLootFrames) do
    frame.loot = nil
    frame:Hide()
  end
  for i,frame in ipairs(playerFrames) do
    frame.player = nil
    frame:Hide()
  end
end

function HLUI:ShowCandidates(winner, candidates)
  if not candidates then return end
  print("HLUI: showing a frame with "..(winner or "no winner").." out of "..#candidates.." candidates")
  local f = candidateFrame
  f:SetPoint("TOPLEFT",controlFrame,"TOPRIGHT", 5, 0)
  f.winner:SetText(winner)
  local tmp = ""
  for i,loser in pairs(candidates) do
    tmp = tmp..loser..((loser==winner) and " *\n" or "\n")
  end
  f.losers:SetText(tmp)
  f:SetHeight(40 + (#candidates)*10)
	f:Show()
end

function HLUI:SetHelp(text)
  if not text then return end
  helpAnchor.label:SetText(text)
end

helpAnchor:RegisterEvent("ADDON_LOADED")
helpAnchor:SetScript("OnEvent", function(frame, event, addon)
	-- if addon ~= "teksLoot" then return end

	helpAnchor:UnregisterEvent("ADDON_LOADED")

	if not hazLootDB then hazLootDB = {} end
	hazLootDB.db = hazLootDB
	helpAnchor:SetPoint("CENTER", UIParent, hazLootDB.db.x and "BOTTOMLEFT" or "BOTTOM", hazLootDB.db.x or 0, hazLootDB.db.y or 800)
end)

-- TODO: Move to /hl command
SlashCmdList["HLUI"] = function() if helpAnchor:IsVisible() then helpAnchor:Hide() else helpAnchor:Show() end end
SLASH_HLUI1 = "/hlui"
