require("HSK")
local tmp, tmpN, tmpTable, tmpIndex

function HSK:TestCapitalize()
  -- Capitalize
  assert("Unit" == HSK:Capitalize("unit"), "Capitalization failed")
  assert("Unit" == HSK:Capitalize("UNIT"), "Capitalization failed")
  assert("Unit" == HSK:Capitalize("UnIt"), "Capitalization failed")
  return true
end

function HSK:TestSetRoles()
  -- SetRoles
  HSK:Print("Testing SetRoles()")
  assert(not HSK:SetRoles(), "SetRoles should fail without a role table")
  tmpN = HSK:SetRoles(Test.Roles)
  assert(tmpN == (#Test.Roles), "SetRoles should return "..(#Test.Roles).." roles in set, got: "..(tmpN or "nil"))
  return true
end

function HSK:TestAddMember()
  -- AddMember
  HSK:Print("Testing AddMember()")
  assert(not HSK:AddMember(), "AddMember should return nil with no arguments")
  assert(not HSK:AddMember(Test.Mains[1]), "AddMember should return nil with no role argument")
  assert(not HSK:AddMember(Test.Mains[1],Test.BadRole), "AddMember should return nil with bad role argument")
  assert(not HSK:AddMember(Test.Mains[1],Test.Roles[1],0), "AddMember should return nil with position <= 0")
  assert(not HSK:AddMember(Test.Mains[1],Test.Roles[1],-10), "AddMember should return nil with position <= 0")
  tmpN = HSK:GetMasterListSize()
  assert(0 == tmpN, "The Master list should still be empty, but is "..(tmpN or "nil"))
  tmpN = HSK:GetPresentCount()
  assert(0 == tmpN, "The Present list should still be empty, but is "..(tmpN or "nil"))

  tmp = HSK:AddMember(Test.Mains[1],Test.Roles[1])
  assert(tmp == 1, "The first member should be in position 1, but is "..(tmp and "true" or "nil"))
  tmp = HSK:GetRole(Test.Mains[1])
  assert(tmp == Test.Roles[1], Test.Mains[1].." should have raid role \""..Test.Roles[1].."\", but has "..(tmp and "true" or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[1])
  assert(tmpN == 1, "Should have 1 member with role \""..Test.Roles[1].."\" but have "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 1, Test.Mains[1].." should have 1 toon, but has "..(tmpN or "nil"))

  tmp = HSK:AddMember(Test.Mains[1],Test.Roles[1])
  assert(tmp == 1, "The first member should still be in position 1, but is "..(tmp and "true" or "nil"))
  assert(HSK:GetMasterListSize() == 1, "The Master list should still have only 1 member, but has "..(tmp and "true" or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[1])
  assert(tmpN == 1, "Should have 1 member with role \""..Test.Roles[1].."\" but have "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 1, Test.Mains[1].." should have 1 toon, but has "..(tmpN or "nil"))

  tmp = HSK:AddMember(Test.Mains[2],Test.Roles[2],3)
  assert(tmp == 2, "The second member should be in position 2, but is "..(tmp and "true" or "nil"))
  assert(HSK:GetMasterListSize() == 2, "The Master list should now have 2 members, but has "..(tmp and "true" or "nil"))
  assert(HSK:GetPresentCount() == 0, "The Present list still be empty, but has non-zero size")
  assert(1 == HSK:GetMasterPosition(Test.Mains[1]), Test.Mains[1].." should still be in position 1")
  assert(2 == HSK:GetMasterPosition(Test.Mains[2]), Test.Mains[2].." should still be in position 2")
  tmpN = HSK:GetRoleCount(Test.Roles[1])
  assert(tmpN == 1, "Should have 1 member with role \""..Test.Roles[1].."\" but have "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[2])
  assert(tmpN == 1, "Should have 1 member with role \""..Test.Roles[2].."\" but have "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 1, Test.Mains[1].." should have 1 toon, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[2])
  assert(tmpN == 1, Test.Mains[2].." should have 1 toon, but has "..(tmpN or "nil"))

  tmp = HSK:AddMember(Test.Mains[3],Test.Roles[3],2)
  assert(2 == tmp, Test.Mains[3].." should now be at position 2, but is "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should still be in position 1, but is "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[2])
  assert(3 == tmpN, Test.Mains[2].." should now be in position 3, but is "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 1, Test.Mains[1].." should have 1 toon, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[2])
  assert(tmpN == 1, Test.Mains[2].." should have 1 toon, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 1, Test.Mains[3].." should have 1 toon, but has "..(tmpN or "nil"))
  HSK:Print("*** AddMember() seems to work as expected.")
end

function HSK:TestRemoveMember()
  --- RemoveMember
  HSK:Print("Testing RemoveMember()")
  tmp = HSK:RemoveMember(Test.Mains[3])
  assert(Test.Mains[3] == tmp, "Should have got main name to confirm, instead got: "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(2 == tmpN, Test.Mains[3].." should still be at position 2, but is "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[3])
  assert(tmpN == 2, "Should still have 2 members with role \""..Test.Roles[3].."\" but has "..(tmpN or "nil"))
  tmp = HSK:RemoveMember(Test.Mains[3], true)
  assert(true == tmp, Test.Mains[3].." should have been removed, instead returned "..(tmp and "true" or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 0, Test.Mains[3].." should now have 0 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Alts[3])
  assert(tmpN == 0, Test.Alts[3].." should also have 0 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(nil == tmpN, Test.Mains[3].." should have nil master position, not "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Alts[3])
  assert(nil == tmpN, Test.Alts[3].." should have nil master position, not "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[3])
  assert(tmpN == 0, "Should have 0 members with role \""..Test.Roles[3].."\" but has "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should still be in position 1, but is "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[2])
  assert(2 == tmpN, Test.Mains[2].." should now be back in position 2, but is "..(tmpN or "nil"))
  tmp = HSK:AddMember(Test.Mains[3],Test.Roles[3],2)

  tmp = HSK:RemoveMember(2)
  assert(Test.Mains[3] == tmp, "Should have got main name to confirm, instead got: "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(2 == tmpN, Test.Mains[3].." should now be at position 2, but is "..(tmpN or "nil"))
  tmp = HSK:RemoveMember(2,true)
  assert(true == tmp, "Failed to remove 2nd member, "..Test.Mains[3])
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(nil == tmpN, Test.Mains[3].." should have nil master position, not "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should still be in position 1, but is "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[2])
  assert(2 == tmpN, Test.Mains[2].." should now be back in position 2, but is "..(tmpN or "nil"))

  HSK:AddMember(Test.Mains[3],Test.Roles[3],2)
  HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  tmp = HSK:RemoveMember(Test.Alts[3])
  assert(Test.Mains[3] == tmp, "Should have got main name to confirm, instead got: "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Alts[3])
  assert(2 == tmpN, Test.Alts[3].." should now be at position 2, but is "..(tmpN or "nil"))
  tmp = HSK:RemoveMember(Test.Alts[3], true)
  assert(true == tmp, Test.Mains[3].." should have been removed, instead returned "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(nil == tmpN, Test.Mains[3].." should have nil master position, not "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should still be in position 1, but is "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[2])
  assert(2 == tmpN, Test.Mains[2].." should now be back in position 2, but is "..(tmpN or "nil"))
  HSK:Print("*** RemoveMember() seems to work as expected.")
end

function HSK:TestAddAlt()
  --- AddAlt
  HSK:Print("Testing AddAlt()")
  tmp = HSK:AddAlt("nonexistent",""..Test.Alts[1].."")
  assert(nil == tmp, "AddAlt should have failed with a nonexistent main, but returned "..(tmp and "true" or "nil"))
  tmpN = HSK:AddAlt(Test.Mains[1],Test.Alts[1])
  assert(1 == tmpN, "Should have got position 1 for AddAlt, but got "..(tmpN or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Alts[1])
  assert(1 == tmpN, Test.Alts[1].." was not in position 1, but "..(tmpN or "nil"))
  tmp = HSK:GetMain(Test.Alts[1])
  assert(Test.Mains[1] == tmp, Test.Mains[1].." should be the main of "..Test.Alts[1]..", but got: "..(tmp and "true" or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[1])
  assert(tmpN == 2, Test.Mains[1].." should now have 2 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[2])
  assert(tmpN == 1, Test.Mains[2].." should have 1 toon, but has "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 1, Test.Mains[3].." should have 1 toon, but has "..(tmpN or "nil"))
  tmp = HSK:GetRole(Test.Alts[1])
  assert(Test.Roles[1] == tmp, Test.Alts[1].." should have defaulted to the same role as "..Test.Mains[1].." but got "..(tmp and "true" or "nil"))
  tmp = HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 2, Test.Mains[3].." should have 2 toons, but has "..(tmpN or "nil"))
  tmp = HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 2, Test.Mains[3].." should still have 2 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount(Test.Roles[3])
  assert(tmpN == 2, "Should have 2 members with role \""..Test.Roles[3].."\" but has "..(tmpN or "nil"))
  tmp = HSK:AddAlt(Test.Mains[2],Test.Alts[3])
  assert(nil == tmp, "Should not have been able to add "..Test.Alts[3].." to "..Test.Mains[2]..", since it already exists on "..Test.Mains[3]..", got: "..(tmp and "true" or "nil"))
  tmp = HSK:GetMain(Test.Mains[3])
  assert(Test.Mains[3] == tmp, "Main of "..Test.Mains[3].." should be "..Test.Mains[3]..", but is "..(tmp and "true" or "nil"))
  tmp = HSK:GetMain(Test.Alts[3])
  assert(Test.Mains[3] == tmp, "Main of "..Test.Alts[3].." should be "..Test.Mains[3]..", but is "..(tmp and "true" or "nil"))
  HSK:Print("*** AddAlt() seems to work as expected.")
end

--- run tests
function HSK:Test()
  HSK:Debug("Starting SK List tests")
  tmpN = HSK:GetMasterListSize()
  assert(0 == tmpN, "The Master list should still be empty, but is "..(tmpN or "nil"))
  tmpN = HSK:GetPresentCount()
  assert(0 == tmpN, "The Present list should still be empty, but is "..(tmpN or "nil"))

  HSK:TestCapitalize()
  HSK:TestSetRoles()
  HSK:TestAddMember()
  HSK:TestAddAlt()
  HSK:TestRemoveMember()

  --- RemoveAlt
  HSK:Print("Testing RemoveAlt()")
  HSK:AddMember(Test.Mains[3],Test.Roles[3],2)
  HSK:AddAlt(Test.Mains[3],Test.Alts[3])
  HSK:AddAlt(Test.Mains[3],Test.Alts[4])
  HSK:AddAlt(Test.Mains[3],Test.Alts[5])
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 4, Test.Mains[3].." should have 4 toons before removal, but has "..(tmpN or "nil"))
  tmp = HSK:RemoveAlt(Test.Mains[3]) -- false
  assert(false == tmp, Test.Mains[3].." should have returned false: it's a main, instead got "..(tmp and "true" or "nil"))
  tmp = HSK:RemoveAlt("nosuchalt") -- nil
  assert(nil == tmp, "'nosuchalt' should have returned nil: it doesn't exist; instead got "..(tmp and "true" or "nil"))
  tmp = HSK:RemoveAlt(Test.Alts[3]) -- true
  assert(true == tmp, "Should have returned 'true' removing "..Test.Alts[3].." instead got "..(tmp and "true" or "nil"))
  tmp = HSK:RemoveAlt(Test.Alts[3]) -- nil
  assert(nil == tmp, Test.Alts[3].." should have returned nil: it doesn't exist; instead got "..(tmp and "true" or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[3])
  assert(2 == tmpN, Test.Mains[3].." should still be in position 2, but is "..(tmpN or "nil"))
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 3, Test.Mains[3].." should have 3 toons, but has "..(tmpN or "nil"))
  HSK:Print("*** RemoveAlt() seems to work as expected.")

  --- PromoteAlt
  HSK:Print("Testing PromoteAlt()")
  HSK:AddAlt(Test.Mains[1],Test.Alts[1])
  HSK:AddAlt(Test.Mains[3],Test.Alts[3], Test.Roles[2])
  tmpN = HSK:GetToonCount(Test.Mains[3])
  assert(tmpN == 4, Test.Mains[3].." should have 4 toons, but has "..(tmpN or "nil"))
  tmpN = HSK:PromoteAlt()
  assert(nil == tmpN, "Promoting nothing should fail, but returned "..(tmpN or "nil"))
  tmp = HSK:ExportMasterList()
  tmpN = HSK:PromoteAlt(Test.Mains[3])
  assert(tmp == HSK:ExportMasterList(), "Export list should be unchanged")
  assert(2 == tmpN, "Promoting "..Test.Mains[3].." should return position 2, but returned "..(tmpN or "nil"))
  tmpN = HSK:PromoteAlt(Test.Alts[3])
  assert(tmpN == HSK:GetMasterListSize(), "Promoting "..Test.Alts[4].." should return position "..HSK:GetMasterListSize()..", but returned "..(tmpN or "nil"))
  HSK:SetDropOnMainChange(false)
  tmpN = HSK:PromoteAlt(Test.Alts[4])
  assert(tmpN == HSK:GetMasterListSize(), "Promoting "..Test.Alts[4].." should return position 2, but returned "..(tmpN or "nil"))
  tmpN = HSK:PromoteAlt(Test.Mains[3])
  HSK:SetDropOnMainChange(true)
  assert(tmpN == HSK:GetMasterListSize(), "Promoting "..Test.Mains[3].." should return position "..HSK:GetMasterListSize()..", but returned "..(tmpN or "nil"))
  HSK:Print("*** PromoteAlt() seems to work as expected")

  --- Clear
  HSK:Print("Testing Clear()")
  assert(HSK:Clear(), "Clear failed. WTF?")
  tmpN = HSK:GetMasterListSize()
  assert(0 == tmpN, "The Master list should still be empty, but is "..(tmpN or "nil"))
  tmpN = HSK:GetPresentCount()
  assert(0 == tmpN, "The Present list should still be empty, but is "..(tmpN or "nil"))
  HSK:Print("*** Clear() seems to work as expected")

  --- Bulk add
  HSK:Print("Testing SetMasterList()")
  tmpN = HSK:SetMasterList(Test.BulkMains)
  assert(10 == tmpN, "Should have 10 new members, instead have "..(tmpN or "nil"))
  tmpN = HSK:GetRoleCount("R1")
  assert(4  == tmpN, "Should have 4 members of R1, intead have "..(tmpN or "nil"))
  for i=1,10 do
    tmpN = HSK:GetMasterPosition("Main"..i)
    assert(i == tmpN, "Main"..i.." should be in position "..i..", instead it's in "..(tmpN or "nil"))
    tmp = HSK:GetRole("Main"..i) -- next assertion is ugly thanks to 1-based tables.
    assert("R"..((i-1)%3+1) == tmp, "Main"..i.." should have role R"..((i-1)%3+1)..", instead it's "..(tmp or "nil"))
  end
  HSK:Clear()
  tmpN = HSK:SetMasterList(Test.BulkMains, Test.BulkAlts)
  assert(10 == tmpN, "Should have 10 new members, instead have "..(tmpN or "nil"))
  tmpTable = HSK:GetAlts(Test.Mains[1])
  assert(nil ~= tmpTable, "tmpTable is nil. It shouldn't be.")
  assert(2 == #tmpTable, Test.Mains[1].." should have 2 alts, but has "..(#tmpTable))
  tmpTable = HSK:GetAlts(Test.Mains[2])
  assert(0 == #tmpTable, Test.Mains[2].." should have 0 alts, but has "..(#tmpTable))
  tmpTable = HSK:GetAlts(Test.Mains[3])
  assert(3 == #tmpTable, Test.Mains[3].." should have 3 alts, but has "..(#tmpTable))
  tmpTable = HSK:GetAlts(Test.Mains[4])
  assert(0 == #tmpTable, Test.Mains[4].." should have 0 alts, but has "..(#tmpTable))
  HSK:Print("*** SetMasterList() seems to work as expected")

  --- GetMasterList()
  HSK:Print("Testing GetMasterList()")
  tmp = HSK:ExportMasterList()
  HSK:SetMasterList(HSK:GetMasterList())
  assert(tmp == HSK:ExportMasterList(), "Importing exported master list shouldn't change it")
  HSK:Print("*** GetMasterList() seems to work as expected")

  --- MarkAsPresent()
  HSK:Print("Testing MarkAsPresent()")
  tmpN = HSK:MarkAsPresent(Test.BadMain)
  assert (nil == tmpN, "Should not have been able to mark a non-existent main as present")
  tmpN = HSK:MarkAsPresent(Test.Alts[4])
  assert(HSK:IsPresent(Test.Alts[4]), Test.Alts[4].." should give 'true' for IsPresent()")
  assert ("Main3" == tmpN, "Expected "..Test.Alts[4].." to be present at 3, but got "..(tmpN or "nil"))
  tmpN = HSK:MarkAsPresent(Test.Mains[4])
  assert ("Main4" == tmpN, "Expected "..Test.Mains[4].." to be present at 4, but got "..(tmpN or "nil"))
  tmpN = HSK:MarkAsPresent(Test.Mains[7])
  tmpN = HSK:MarkAsPresent(Test.Mains[8])
  tmpN = HSK:GetPresentCount()
  assert(4 == tmpN, "Was expecting 4 members present, but got "..(tmpN or "nil"))
  HSK:Print("*** MarkAsPresent() seems to work as expected")

  --- IsPresent()
  HSK:Print("Testing IsPresent()")
  assert(HSK:IsPresent(Test.Alts[4]), Test.Alts[4].." should give 'true' for IsPresent()")
  assert(HSK:IsPresent(string.lower(Test.Alts[4])), string.lower(Test.Alts[4]).." should give 'true' for IsPresent()")
  HSK:Print("*** IsPresent() seems to work as expected")

  --- Suicide()
  HSK:Print("Testing Suicide()")
  tmpN = HSK:Suicide(Test.Alts[4])
  assert(8 == tmpN, "Was expecting "..Test.Alts[4].." to be at the bottom of the present list, but got "..(tmpN or "nil")) 
  for i,main in ipairs({"Main1","Main2","Main4","Main7","Main5","Main6","Main8","Main3","Main9","Main10",}) do
    tmp = HSK:GetMemberAt(i)
    assert(main == tmp, "Expected "..main.." at position "..i..", but got "..(tmp or "nil"))
  end
  tmpN = HSK:Suicide(Test.Mains[8])
  assert(8 == tmpN, "Was expecting "..Test.Mains[8].." to be at the bottom of the present list, but got "..(tmpN or "nil")) 
  for i,main in ipairs({"Main1","Main2","Main4","Main7","Main5","Main6","Main3","Main8","Main9","Main10",}) do
    tmp = HSK:GetMemberAt(i)
    assert(main == tmp, "Expected "..main.." at position "..i..", but got "..(tmp or "nil"))
  end
  HSK:Print("*** Suicide() seems to work as expected")

  -- ChangeRole()
  HSK:Print("Testing ChangeRole()")
  tmp = HSK:ChangeRole(Test.Mains[1],HSK:GetRole(Test.Mains[1]))
  assert ("R1" == tmp, "Was expecting role to stay the same, but it's now: "..(tmp or "nil"))
  tmpN = HSK:GetMasterPosition(Test.Mains[1])
  assert(1 == tmpN, Test.Mains[1].." should be in position 1, instead it's in "..(tmpN or "nil"))
  assert ("R1" == tmp, "Was expecting to change to R1, but it's : "..(tmp or "nil"))
  tmp = HSK:ChangeRole(Test.Mains[2],HSK:GetRole(Test.Mains[1]))
  for i,main in ipairs({"Main1","Main4","Main7","Main5","Main6","Main3","Main8","Main9","Main10","Main2",}) do
    tmp = HSK:GetMemberAt(i)
    assert(main == tmp, "Expected "..main.." at position "..i..", but got "..(tmp or "nil"))
  end
  HSK:SetDropOnRoleChange(false)
  tmp = HSK:ChangeRole(Test.Mains[4],"R2")
  for i,main in ipairs({"Main1","Main4","Main7","Main5","Main6","Main3","Main8","Main9","Main10","Main2",}) do
    tmp = HSK:GetMemberAt(i)
    assert(main == tmp, "Expected "..main.." at position "..i..", but got "..(tmp or "nil"))
  end
  HSK:Print("*** ChangeRole() seems to work as expected")

  -- GetCandidate()
  HSK:Print("Testing GetCandidate()")
  assert(not HSK:GetCandidate(), "Expected nil return from GetCandidate(nil)")
  tmpN = HSK:GetPresentCount()
  assert(4 == tmpN, "Expected 4 present, found: "..(tmpN or "nil"))
  tmp = HSK:GetCandidate({"Main6","Main8","Main7",})
  assert("Main7" == tmp, "Expected Main7 to be the winner of Main6,Main7,Main8, got: "..(tmp or "nil"))
  tmp = HSK:GetCandidate({"main6","main8","main7",})
  assert("Main7" == tmp, "Expected Main7 to be the winner of main6,main7,main8")
  tmp = HSK:GetCandidate({"mAiN6","mAiN8","mAiN7",})
  assert("Main7" == tmp, "Expected Main7 to be the winner of MAiN6,MAiN7,MAiN8")

  HSK:Debug("Tests completed successfully!")

  HSK:Print("Getting present list for visual check:")
  tmp = HSK:GetPresentList()
  HSK:Print("Exporting present list for visual check:")
  print(HSK:ExportPresentList())
  HSK:Print("Exporting master list for visual check:")
  print(HSK:ExportMasterList())
end

-------- TEST DATA -------
Test = {}
Test.Roles   = {"R1","R2","R3",}
Test.Mains   = {"Main1","Main2","Main3","Main4","Main5","Main6","Main7","Main8","Main9","Main10",}
Test.Alts    = {"Main1Alt1","Main1Alt2","Main3Alt1","Main3Alt2","Main3Alt3",}
Test.BulkMains = {"Main1","R1","Main2","R2","Main3","R3","Main4","R1","Main5","R2","Main6","R3","Main7","R1","Main8","R2","Main9","R3","Main10","R1",}
Test.BulkAlts  = {["Main1"]={"Main1Alt1","R1","Main1Alt2","R2",},["Main3"]={"Main3Alt1","R1","Main3Alt2","R2","Main3Alt3","R3",},}
Test.BadRole = "R0"
Test.BadMain = "Main0"
Test.BadAlt  = "Main0Alt0"
-------- MAIN -------
HSK:Test()
